import { Component, Injectable } from '@angular/core';
import { Location,DatePipe } from '@angular/common';
import { Store } from '@ngrx/store';
import * as mapReducers from '../../reducers/map.reducer';
import { commonReducers, ItemTypeService, ItemService, FolderService,IItem,TimespanService } from '@farmmaps/common';
import { Router } from '@angular/router';
import { ForItemType } from '../for-item/for-itemtype.decorator';
import { AbstractSelectedItemComponent } from '../selected-item/selected-item.component';
import { ITemporalItemLayer} from '../../models/item.layer';
import * as mapActions from '../../actions/map.actions';


@ForItemType("vnd.farmmaps.itemtype.temporal")
@Injectable()
@Component({
  selector: 'fm-map-selected-item-temporal',
  templateUrl: './selected-item-temporal.component.html',
  styleUrls: ['./selected-item-temporal.component.scss']
})
export class SelectedItemTemporalComponent extends AbstractSelectedItemComponent {

  constructor(store: Store<mapReducers.State | commonReducers.State>, itemTypeService: ItemTypeService, location: Location, router: Router, private itemService$: ItemService,private folderService$: FolderService, private timespanService$:TimespanService,private datePipe$: DatePipe) {
    super(store, itemTypeService,location,router);
  }

  onLayerChanged(layerIndex: number) {
    this.store.dispatch(new mapActions.SetLayerIndex(layerIndex));
  }

  hasNext():boolean {
    let temporalItemLayer = this.itemLayer as  ITemporalItemLayer;
    return temporalItemLayer && temporalItemLayer.nextItemLayer != null;
  }

  selectedDate():string {
    let temporalItemLayer = this.itemLayer as  ITemporalItemLayer;
    if(temporalItemLayer.selectedItemLayer) {
      if(temporalItemLayer.selectedItemLayer.item.dataDate && temporalItemLayer.selectedItemLayer.item.dataEndDate) {
        let sd = new Date(Date.parse(temporalItemLayer.selectedItemLayer.item.dataDate));
        let ed = new Date(Date.parse(temporalItemLayer.selectedItemLayer.item.dataEndDate));
        return this.timespanService$.getCaption(sd,ed);
      } else {
        let d = new Date(Date.parse(temporalItemLayer.selectedItemLayer.item.dataDate));
        return this.datePipe$.transform(d, "shortDate");  
      }
    }
    return "";
  }

  nextDate():string {
    let temporalItemLayer = this.itemLayer as  ITemporalItemLayer;
    if(temporalItemLayer.nextItemLayer.item) {
      if(temporalItemLayer.nextItemLayer.item.dataDate && temporalItemLayer.nextItemLayer.item.dataEndDate) {
        let sd = new Date(Date.parse(temporalItemLayer.nextItemLayer.item.dataDate));
        let ed = new Date(Date.parse(temporalItemLayer.nextItemLayer.item.dataEndDate));
        return this.timespanService$.getCaption(sd,ed);
      } else {
        let d = new Date(Date.parse(temporalItemLayer.nextItemLayer.item.dataDate));
        return this.datePipe$.transform(d, "shortDate");  
      }
    }
    return "";
  }

  hasPrevious():boolean {
    let temporalItemLayer = this.itemLayer as  ITemporalItemLayer;
    return temporalItemLayer && temporalItemLayer.previousItemLayer != null;
  }

  previousDate():string {
    let temporalItemLayer = this.itemLayer as  ITemporalItemLayer;
    if(temporalItemLayer.previousItemLayer.item)  {
      if(temporalItemLayer.previousItemLayer.item.dataDate && temporalItemLayer.previousItemLayer.item.dataEndDate) {
        let sd = new Date(Date.parse(temporalItemLayer.previousItemLayer.item.dataDate));
        let ed = new Date(Date.parse(temporalItemLayer.previousItemLayer.item.dataEndDate));
        return this.timespanService$.getCaption(sd,ed);
      } else {
        let d = new Date(Date.parse(temporalItemLayer.previousItemLayer.item.dataDate));
        return this.datePipe$.transform(d, "shortDate");  
      }
    }
    return "";
  }

  handleNextTemporal(event:MouseEvent) {
    this.store.dispatch(new mapActions.NextTemporal());
    event.preventDefault();    
  }

  handlePreviousTemporal(event:MouseEvent) {
    this.store.dispatch(new mapActions.PreviousTemporal());
    event.preventDefault();
  }

  selectedItem():IItem {
    let temporalItemLayer = this.itemLayer as ITemporalItemLayer;
    if(temporalItemLayer && temporalItemLayer.selectedItemLayer) {
       return temporalItemLayer.selectedItemLayer.item;
    }
    return null;
  }  

  layer(layers:any,layerIndex:number) {
    return layers.find(l => l.index == layerIndex);
  }

  handleGoToChart(item: IItem) {  
    this.router.navigate(['/viewer', 'temporal', 'item', item.parentCode, new Date(Date.parse(item.dataDate)).getUTCFullYear()]);
    return false;
  }
}
